<footer class="footer">
    <div class="container">
        <div class="has-text-centered">
            <p><a href="https://codeberg.org/hjacobs/kube-web-view">Kubernetes Web View</a> {{ version }}</p>
        </div>
    </div>
</footer>
