/* */
html { overflow: scroll; }

.navbar-end { flex-grow: 0.33; }
.navbar-end .navbar-item { width: 100%; }
.navbar-end form { width: 100%; }
.navbar-end input { color: #fff; background: transparent; box-shadow: none; border: none; border-radius: 0; border-bottom: 1px solid #98b8ed;}
.navbar-end input::placeholder { color: #98b8ed; }
.navbar-end input:focus { border-color: #fff; }
.navbar-end .control .icon { color: #98b8ed; }
.navbar-end .control .icon { color: #98b8ed; }

.container.is-fluid { margin-left: 0; padding: 1.5rem 0; }


/* aside menu has toggle for mobile */
aside.menu { padding: 1.5rem; margin-top: -1.5rem; background: #fafafa; }

main { flex-grow: 1; flex-shrink: 1; }

h1 .links,
h2 .links {
    margin: 0 1rem;
}
h1 { display: flex; }
h1 .meta,
h2 .meta {
    margin-left: auto;
    font-size: 1rem;
    font-weight: normal;
    color: #7a7a7a;
    padding-top: 0.5rem;
}

/* desktop+ */
@media screen and (min-width: 1024px) {
    aside.menu { flex-grow: 0; flex-shrink: 0; flex-basis: 12rem; padding: 1.5rem; margin-right: 1.5rem; margin-top: -1.5rem; }
}
/* mobile & touch */
@media screen and (max-width: 1023px) {

    /* mobile menu has white background, so change our text color */
    .navbar-end input { color: #363636; }

    aside.menu { padding: 0.5rem 1.5rem; margin-bottom: 1.5rem; }
    aside #aside-menu { display: none; padding-top: 1rem; }

    aside a[role=button] { display: block; height: 1rem;}
    aside a[role=button] span { display: block; background: hsl(0, 0%, 71%); position: relative; height: 1px; margin: 0 auto; width: 66%; }
    aside a[role=button] span:nth-child(1) { top: calc(50% - 1px); }
    aside a[role=button] span:nth-child(2) { top: calc(50% + 1px); }

    #aside-menu.is-active { display: block; }

    main { padding: 0 1.5rem; }

    /* show meta created below title */
    h1 { display: block; }
    h1 .meta,
    h2 .meta {
        margin-left: 0;
        display: block;
    }
}

table th a { color: #363636; }
table th a:hover { color: #3273dc; }

table th.num:not([align]),
table td.num:not([align]) { text-align: right; }

table.table.has-bottom-controls { margin-bottom: 0.5rem; }
.bottom-controls { margin-bottom: 1.5rem; }

div.section { margin: 0 0 1rem 0; padding: 0; }
div.section h4.title { margin-bottom: 1rem; }

a.toggle-tools.is-active span {
    transform: rotate(180deg);
}
form.tools-form { display: none; margin-bottom: 1rem; }
form.tools-form.is-active { display: block; }
form .checkboxes { }
form .checkboxes label { margin-right: 0.5rem; padding-top: 0.375em; }

.search-result p { font-size: 0.75rem; }
.search-result h3.title { margin: 0 0 0.5rem 0; }
.search-result { margin-bottom: 1.5rem; }
.search-result p { margin-bottom: 0.25rem; }
.search-result em { font-style: normal; background: hsl(48, 100%, 67%) }

/* Pygments CSS
 * print(HtmlFormatter(style='friendly').get_style_defs('.highlight'))
 */
.highlight .hll { background-color: #ffffcc }
.highlight  { background: #f0f0f0; }
.highlight .c { color: #60a0b0; font-style: italic } /* Comment */
.highlight .err { border: 1px solid #FF0000 } /* Error */
.highlight .k { color: #007020; font-weight: bold } /* Keyword */
.highlight .o { color: #666666 } /* Operator */
.highlight .ch { color: #60a0b0; font-style: italic } /* Comment.Hashbang */
.highlight .cm { color: #60a0b0; font-style: italic } /* Comment.Multiline */
.highlight .cp { color: #007020 } /* Comment.Preproc */
.highlight .cpf { color: #60a0b0; font-style: italic } /* Comment.PreprocFile */
.highlight .c1 { color: #60a0b0; font-style: italic } /* Comment.Single */
.highlight .cs { color: #60a0b0; background-color: #fff0f0 } /* Comment.Special */
.highlight .gd { color: #A00000 } /* Generic.Deleted */
.highlight .ge { font-style: italic } /* Generic.Emph */
.highlight .gr { color: #FF0000 } /* Generic.Error */
.highlight .gh { color: #000080; font-weight: bold } /* Generic.Heading */
.highlight .gi { color: #00A000 } /* Generic.Inserted */
.highlight .go { color: #888888 } /* Generic.Output */
.highlight .gp { color: #c65d09; font-weight: bold } /* Generic.Prompt */
.highlight .gs { font-weight: bold } /* Generic.Strong */
.highlight .gu { color: #800080; font-weight: bold } /* Generic.Subheading */
.highlight .gt { color: #0044DD } /* Generic.Traceback */
.highlight .kc { color: #007020; font-weight: bold } /* Keyword.Constant */
.highlight .kd { color: #007020; font-weight: bold } /* Keyword.Declaration */
.highlight .kn { color: #007020; font-weight: bold } /* Keyword.Namespace */
.highlight .kp { color: #007020 } /* Keyword.Pseudo */
.highlight .kr { color: #007020; font-weight: bold } /* Keyword.Reserved */
.highlight .kt { color: #902000 } /* Keyword.Type */
.highlight .m { color: #40a070 } /* Literal.Number */
.highlight .s { color: #4070a0 } /* Literal.String */
.highlight .na { color: #4070a0 } /* Name.Attribute */
.highlight .nb { color: #007020 } /* Name.Builtin */
.highlight .nc { color: #0e84b5; font-weight: bold } /* Name.Class */
.highlight .no { color: #60add5 } /* Name.Constant */
.highlight .nd { color: #555555; font-weight: bold } /* Name.Decorator */
.highlight .ni { color: #d55537; font-weight: bold } /* Name.Entity */
.highlight .ne { color: #007020 } /* Name.Exception */
.highlight .nf { color: #06287e } /* Name.Function */
.highlight .nl { color: #002070; font-weight: bold } /* Name.Label */
.highlight .nn { color: #0e84b5; font-weight: bold } /* Name.Namespace */
.highlight .nt { color: #062873; font-weight: bold } /* Name.Tag */
.highlight .nv { color: #bb60d5 } /* Name.Variable */
.highlight .ow { color: #007020; font-weight: bold } /* Operator.Word */
.highlight .w { color: #bbbbbb } /* Text.Whitespace */
.highlight .mb { color: #40a070 } /* Literal.Number.Bin */
.highlight .mf { color: #40a070 } /* Literal.Number.Float */
.highlight .mh { color: #40a070 } /* Literal.Number.Hex */
.highlight .mi { color: #40a070 } /* Literal.Number.Integer */
.highlight .mo { color: #40a070 } /* Literal.Number.Oct */
.highlight .sa { color: #4070a0 } /* Literal.String.Affix */
.highlight .sb { color: #4070a0 } /* Literal.String.Backtick */
.highlight .sc { color: #4070a0 } /* Literal.String.Char */
.highlight .dl { color: #4070a0 } /* Literal.String.Delimiter */
.highlight .sd { color: #4070a0; font-style: italic } /* Literal.String.Doc */
.highlight .s2 { color: #4070a0 } /* Literal.String.Double */
.highlight .se { color: #4070a0; font-weight: bold } /* Literal.String.Escape */
.highlight .sh { color: #4070a0 } /* Literal.String.Heredoc */
.highlight .si { color: #70a0d0; font-style: italic } /* Literal.String.Interpol */
.highlight .sx { color: #c65d09 } /* Literal.String.Other */
.highlight .sr { color: #235388 } /* Literal.String.Regex */
.highlight .s1 { color: #4070a0 } /* Literal.String.Single */
.highlight .ss { color: #517918 } /* Literal.String.Symbol */
.highlight .bp { color: #007020 } /* Name.Builtin.Pseudo */
.highlight .fm { color: #06287e } /* Name.Function.Magic */
.highlight .vc { color: #bb60d5 } /* Name.Variable.Class */
.highlight .vg { color: #bb60d5 } /* Name.Variable.Global */
.highlight .vi { color: #bb60d5 } /* Name.Variable.Instance */
.highlight .vm { color: #bb60d5 } /* Name.Variable.Magic */
.highlight .il { color: #40a070 } /* Literal.Number.Integer.Long */

