<!DOCTYPE html>
<html lang="en">
    <head>
        <meta charset="utf-8">
        <meta name="viewport" content="width=device-width, initial-scale=1">
        <title>{% block title %}{% endblock %} - Kubernetes Web View</title>
        <link rel="stylesheet" href="/assets/bulma.min.css?v=19.9.1">
        <script defer src="/assets/solid.min.js?v=19.9.1"></script>
        <script defer src="/assets/fontawesome.min.js?v=19.9.1"></script>
        <script src="/assets/kube-web.js?v=19.9.1"></script>
        <link rel="stylesheet" href="/assets/kube-web.css?v=19.9.1" />
        <link rel="shortcut icon" href="/assets/favicon.png">
        <link rel="icon" type="image/png" href="/assets/favicon.png">
        {% include "partials/extrahead.html" %}
    </head>
    <body class="has-navbar-fixed-top">
        {% include "partials/navbar.html" %}
        <div class="container is-fluid is-flex-desktop">
            {% include "partials/sidebar.html" %}
            <main>
            {% block content %}{% endblock %}
            </main>
        </div>
        {% include "partials/footer.html" %}
    </body>
</html>

