<h4 class="title is-5">Events</h4>
<table class="table is-striped">
    <tr>
        <th>Type</th>
        <th>Reason</th>
        <th>Age</th>
        <th>From</th>
        <th>Message</th>
    </tr>
{% for event in events: %}
<tr>
    <td class="{{ get_cell_class("events", "Type", event.obj.type) }}">{{ event.obj.type }}</td>
    <td class="{{ get_cell_class("events", "Reason", event.obj.reason) }}">{{ event.obj.reason }}</td>
    <td style="color:{{ event.obj.lastTimestamp|age_color(days=1) }}">{{ event.obj.lastTimestamp.replace('T', ' ').replace('Z', '') }}</td>
    <td>{{ event.obj.source.component }}</td>
    <td>{{ event.obj.message }}</td>
</tr>
{% else: %}
<tr>
    <td colspan="5">No events found.</td>
</tr>
{% endfor %}
</table>
