SORT = "sort"
SELECTOR = "selector"
FILTER = "filter"
JOIN = "join"
LIMIT = "limit"
HIDDEN_COLUMNS = "hidecols"
LABEL_COLUMNS = "labelcols"
CUSTOM_COLUMNS = "customcols"
DOWNLOAD = "download"
VIEW = "view"
API_VERSION = "api_version"
